/* GNU polyxmass - the massist's program.
   -------------------------------------- 
   Copyright (C) 2000,2001,2002,2003,2004 Filippo Rusconi

   http://www.polyxmass.org

   This file is part of the "GNU polyxmass" project.
   
   The "GNU polyxmass" project is an official GNU project package (see
   www.gnu.org) released ---in its entirety--- under the GNU General
   Public License and was started at the Centre National de la
   Recherche Scientifique (FRANCE), that granted me the formal
   authorization to publish it under this Free Software License.

   This software is free software; you can redistribute it and/or
   modify it under the terms of the GNU  General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU  General Public
   License along with this software; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#include "polyxedit-ui-elemcompos.h"
#include "polyxedit-ui-seqed-wnd.h"


GtkWidget *
polyxedit_elemcompos_wnd_setup (PxmEditCtxt *editctxt)
{
  GtkWidget *window = NULL;
  GtkWidget *widget = NULL;

  GladeXML *xml = NULL;

  gchar *gui_file = NULL;
  gchar *help = NULL;
  
  GPtrArray *acGPA = NULL;
  
  
  g_assert (editctxt != NULL);
  

  gui_file = 
    g_strdup_printf ("%s/polyxedit-compositions.glade", userspec->gladedir);
  
  g_assert (gui_file != NULL);
  
  xml = glade_xml_new (gui_file, "elemental_composition_wnd", 
		       PACKAGE);
  if (xml == NULL)
    {
      g_error (_("%s@%d: failed loading the interface\n"),
	     __FILE__, __LINE__);

      return NULL;
    }
  
  window = glade_xml_get_widget (xml, "elemental_composition_wnd");
  
  if (window == NULL)
    {
      g_critical (_("%s@%d: failed creating the elemental composition window\n"),
	     __FILE__, __LINE__);

      g_object_unref (G_OBJECT (xml));

      return NULL;
    }

  /* Immediately set to the window a pointer to the editctxt:
   */
  g_object_set_data (G_OBJECT (window), "editctxt", editctxt);


  widget = glade_xml_get_widget (xml, "messages_entry");
  g_object_set_data (G_OBJECT (window), "messages_entry",
		     widget);
  

  /* Set the polymer sequence name to its correspondent GtkEntry.
   */
  widget = glade_xml_get_widget (xml, "polymer_sequence_name_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), "polymer_sequence_name_entry",
		     widget);
  
  if (editctxt->polymer->plminfo->name != NULL)
    gtk_entry_set_text (GTK_ENTRY (widget), 
			editctxt->polymer->plminfo->name);
  else
    gtk_entry_set_text (GTK_ENTRY (widget), _("Not set"));
  
  
  /* Set the editctxt pointer to the id number entry.
   */
  widget = glade_xml_get_widget (xml, "polymer_sequence_id_number_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), "polymer_sequence_id_number_entry",
		     widget);
  help = g_strdup_printf ("%p", editctxt);
  gtk_entry_set_text (GTK_ENTRY (widget), help);
  g_free (help);
  
  /* Start/End selection values.
   */
  widget = glade_xml_get_widget (xml, "sel_start_pos_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), "sel_start_pos_entry",
		     widget);

  widget = glade_xml_get_widget (xml, "sel_end_pos_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), "sel_end_pos_entry",
		     widget);
  
  /* The checkbutton that says if the whole sequence should be considered
     or the selection only.
  */
  widget = glade_xml_get_widget (xml, "whole_sequence_checkbutton");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), "whole_sequence_checkbutton",
		     widget);

  /* We have a function that can set up the selection region values into 
     their respective entries:
  */
  polyxedit_elemcompos_wnd_update_sequence_region_button (NULL, window);

  /* The checkbutton that says if we want the computation to be
     incremental or not.
  */
  widget = glade_xml_get_widget (xml, "incremental_checkbutton");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), "incremental_checkbutton",
		     widget);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);

  widget = glade_xml_get_widget (xml, "elemental_composition_entry");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), "elemental_composition_entry",
		     widget);

  /* The update sequence region button.
   */
  widget = glade_xml_get_widget (xml, "update_sequence_region_button");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), 
		     "update_sequence_region_button",
		     widget);
  g_signal_connect 
    (G_OBJECT (widget),
     "clicked",
     G_CALLBACK (polyxedit_elemcompos_wnd_update_sequence_region_button), 
     window);

  /* The execute button.
   */
  widget = glade_xml_get_widget (xml, "elemental_composition_execute_button");
  g_assert (widget != NULL);
  g_object_set_data (G_OBJECT (window), 
		     "elemental_composition_execute_button",
		     widget);
  g_signal_connect (G_OBJECT (widget),
		    "clicked",
		    G_CALLBACK (polyxedit_elemcompos_wnd_execute_button), 
		    window);


  /* We now have to allocate the GPtrArray of atomcount instances
     through which the elemental composition will be calculated. We
     want this GPtrArray to be the propriety of the whole window, and
     not to be localized in the execute_button handler because this
     way it is accessible and remains available if the
     "incremental_checkbutton" widget is active.
  */
  acGPA = g_ptr_array_new ();
  
  /* We know that this array will be populated with PxmAtomcount objects,
     so we know how to free this array when the window is closed.
  */
  g_object_set_data_full (G_OBJECT (window), "acGPA", acGPA,
			  (GDestroyNotify) pxmchem_atomcount_GPA_free);
  
  /* Signal / callback connections.
   */
  g_signal_connect (G_OBJECT (window),
		    "delete_event",
		    G_CALLBACK (polyxedit_elemcompos_wnd_delete_event), 
		    editctxt);
  
  /* Signal / callback connections.
   */
  g_signal_connect (G_OBJECT (window),
		    "destroy_event",
		    G_CALLBACK (polyxedit_elemcompos_wnd_destroy_event), 
		    editctxt);
  
  return window;
}


void
polyxedit_elemcompos_wnd_execute_button (GtkWidget *button,
					  gpointer data)
{
  GtkWidget *window = data;
  GtkWidget *widget = NULL;
  GtkWidget *messages_entry = NULL;
  
  gint len = 0;
  
  GPtrArray *acGPA = NULL;
  
  PxmEditCtxt *editctxt = NULL;

  PxmCompcalcRes res = PXM_COMPCALC_FAILURE;
  
  PxmCalcOpt *calcopt = NULL;
  PxmIonizerule *ionizerule = NULL;
      
  gchar *help = NULL;
  gchar *formula = NULL;
  

  
  g_assert (window != NULL);
  
  
  /* First of all get the array of PxmAtomcount instances.
   */
  acGPA = g_object_get_data (G_OBJECT (window), "acGPA");
  g_assert (acGPA != NULL);
  
  editctxt = g_object_get_data (G_OBJECT (window), "editctxt");
  g_assert (editctxt != NULL);

  g_assert (editctxt->polymer != NULL);
  g_assert (editctxt->polymer->monomerGPA != NULL);
  len = editctxt->polymer->monomerGPA->len;
    
  /* Make a copy of the calcopt/ionizerule instances that belongs to
     the editctxt.
  */
  calcopt = pxmchem_calcopt_dup (editctxt->calcopt);
  ionizerule = pxmchem_ionizerule_dup (editctxt->ionizerule);

  /* We almost certainly will display messages.
   */
  messages_entry = g_object_get_data (G_OBJECT (window), 
				      "messages_entry");
  
  /* We must check if the user wants that the computation be made for
     the whole sequence or for the selection region of the polymer sequence
     only, or for the interval that is defined with the start_pos and 
     end_pos entry widgets.
  */
  widget = 
    g_object_get_data (G_OBJECT (window), "whole_sequence_checkbutton");
  g_assert (widget != NULL);

  if (TRUE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
      calcopt->start_idx = 0;
      calcopt->end_idx = len;
    }
  else
    {
      /* We should check what the start_pos and end_pos entries say:
       */
      widget = g_object_get_data (G_OBJECT (window), "sel_start_pos_entry");
      g_assert (widget != NULL);
      
      help = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
      g_assert (help != NULL);
      
      if (strlen (help) <= 0)
	{
	  calcopt->start_idx = 0;
	}
      else
	{
	  /* There is a string in the entry. Convert it to gint.
	   */
	  if (FALSE == 
	      libpolyxmass_globals_strtoi (help, &calcopt->start_idx, 10))
	    calcopt->start_idx = 0;
	  else
	    {
	      /* Remember that the user "counts" monomers starting at 1,
		 and we need to count starting at 0, because we use indexes
		 and not positions. So decrement by one the value entered
		 by the user.
	      */
	      calcopt->start_idx--;
	    }
	}
      
      widget = g_object_get_data (G_OBJECT (window), "sel_end_pos_entry");
      g_assert (widget != NULL);
      
      help = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
      g_assert (help != NULL);
      
      if (strlen (help) <= 0)
	{
	  calcopt->end_idx = 0;
	}
      else
	{
	  /* There is a string in the entry. Convert it to gint.
	   */
	  if (FALSE == 
	      libpolyxmass_globals_strtoi (help, &calcopt->end_idx, 10))
	    calcopt->end_idx = 0;
	  else
	    {
	      /* Remember that for the end_idx, the computations need
		 to have it incremented by one with respect to the real
		 last monomer index. So, contrary to start_idx, we do not
		 change its value.
		 
		 calcopt->end_idx;
	      */
	    }
	}

      /* Now make some final checks:
       */

      if (calcopt->start_idx > len)
	calcopt->start_idx = len;

      if (calcopt->start_idx < 0)
	calcopt->start_idx = 0;
      
      if (calcopt->end_idx > len)
	calcopt->end_idx = len;

      if (calcopt->end_idx < 0)
	calcopt->end_idx = 0;
    }


  /* If the user wants not that the computations be incremental, 
     we first have to empty the array of atomcounts.
  */
  widget = 
    g_object_get_data (G_OBJECT (window), "incremental_checkbutton");
  g_assert (widget != NULL);
  
  if (FALSE == gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    pxmchem_atomcount_GPA_empty (acGPA);
  
  /* We finally have all needed to perform the computation:
   */
  res = pxmchem_elemcompcalc_polymer (editctxt->polymer,
				      editctxt->polchemdefctxt->polchemdef,
				      calcopt,
				      editctxt->ionizerule, 
				      acGPA,
				      &formula);
  if (res != PXM_COMPCALC_SUCCESS)
    {
      polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
					     _("Computation failed"),
					     POLYXMASS_NORM_MSG_TIMEOUT);
    }
  else
    {
      if (formula == NULL)
	  polyxmass_timeoutmsg_message_set ((GtkWindow *) window,
						 _("Computation failed"),
						 POLYXMASS_NORM_MSG_TIMEOUT);
      else
	{
	  widget = g_object_get_data (G_OBJECT (window), 
				      "elemental_composition_entry");
	  
	  gtk_entry_set_text (GTK_ENTRY (widget), formula);
	  
	  g_free (formula);

	  polyxmass_timeoutmsg_message_set 
	    ((GtkWindow *) window,
	     _("Computation successful"),
	     POLYXMASS_NORM_MSG_TIMEOUT);
	}
    }
  
  /* At this point we can free the helper stuff.
   */
  pxmchem_calcopt_free (calcopt);
  pxmchem_ionizerule_free (ionizerule);
  
  return;
}


void
polyxedit_elemcompos_wnd_update_sequence_region_button (GtkWidget *button,
							gpointer data)
{
  GtkWidget *window = data;
  GtkWidget *widget = NULL;
  
  PxmEditCtxt *editctxt = NULL;

  gchar *help = NULL;

  gint start = 0;
  gint end = 0;
  
  g_assert (window != NULL);

  editctxt = g_object_get_data (G_OBJECT (window), "editctxt");
  
  /* Prepare the variables from the sequence editor window, so that
     we can put them immediately in the start/end positions's entry
     windows.
  */
  if (FALSE != polyxedit_seqed_wnd_get_selection_indices (editctxt,
							  &start, &end))
    {
      /* Remember that due to the handling of the polymer sequence
	 displayed in the sequence editor, the end value that is 
	 returned is indeed corresponding to the real index of the last
	 selected monomer + 1, so care should be taken not to outbound
	 the polymer sequence array.
      */
      
      /* We show the (start + 1) value because we want to show
	 a position and not an index, while the end needs not be
	 incremented because it is by itself already incremented.
      */      
      widget = g_object_get_data (G_OBJECT (window), "sel_start_pos_entry");
      g_assert (widget != NULL);

      help = g_strdup_printf ("%d", start + 1);
      gtk_entry_set_text (GTK_ENTRY (widget), help);
      g_free (help);

      widget =  g_object_get_data (G_OBJECT (window), "sel_end_pos_entry");
      g_assert (widget != NULL);
      help = g_strdup_printf ("%d", end);
      gtk_entry_set_text (GTK_ENTRY (widget), help);
      g_free (help);

      /* If there is a sequence selected, by default we disable the
	 "whole sequence" checkbutton. Otherwise we enable it.
      */
      widget = g_object_get_data (G_OBJECT (window), 
				  "whole_sequence_checkbutton");
      g_assert (widget != NULL);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
    }
  else
    {
      widget = g_object_get_data (G_OBJECT (window), "sel_start_pos_entry");
      g_assert (widget != NULL);

      widget = g_object_get_data (G_OBJECT (window), "sel_end_pos_entry");
      g_assert (widget != NULL);

      /* If there is a sequence selected, by default we disable the
	 "whole sequence" checkbutton. Otherwise we enable it.
      */
      widget = g_object_get_data (G_OBJECT (window), 
				  "whole_sequence_checkbutton");
      g_assert (widget != NULL);
      
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
    }
  
  return;
}


gint 
polyxedit_elemcompos_wnd_delete_event (GtkWidget *window, 
					GdkEventAny *event, 
				       gpointer data)
{
  gint count = 0;
  
  /* 
     Prior to closing the window, we want to make sure that no
     pending timed-out messages are there...
  */
  count = polyxmass_timeoutmsg_messages_remove ((GtkWindow *) window);
  
  /*
    printf ("polyxedit_elemcompos_wnd_delete_event, count is %d\n",
    count);
  */

  return FALSE;
}



gint 
polyxedit_elemcompos_wnd_destroy_event (GtkWidget *window, 
					GdkEventAny *event, 
					gpointer data)
{
  printf ("polyxedit_elemcompos_wnd_destroy_event\n");

  return FALSE;
}


